/*
** Module   : ComboBoxModel.java
** Abstract : Combo-box data model.
**
** Copyright (c) 2010-2015, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- --JPRM-- ----------------Description-----------------
** 001 SIY 20100629          Created initial version
** 002 SIY 20100804          Changes in imports, some refactorings.
** 003 IAS 20150202          Add selected index.
*/

package com.goldencode.p2j.ui.client.model;

import com.goldencode.p2j.ui.*;
import com.goldencode.p2j.util.*;

/**
 * Combo-box data model
 * 
 * @param   <T>
 *          Type of the item stored in the model. 
 */
public class ComboBoxModel<T>
extends ListModel<T>
{
   /** Selected item. */
   private T selected;
   /** Selected index. */
   private int selectedIndex = -1;
   /** Selected item id. */
   private int selectedId;
   /** Selected value. */
   private BaseDataType selectedValue;

   /**
    * Add new item at the end of model.
    * 
    * @param   item
    *          Item to add.
    */
   @Override
   public void add(T item)
   {
      super.add(item);

      if (size() == 1)
         selected = item;
   }

   /**
    * Insert item at specified location.
    *  
    * @param   item
    *          Item to insert.
    * @param   index
    *          Index at which item should be inserted.
    */
   public void insert(T item, int index)
   {
      add(index, item);

      if (size() == 1)
         selected = item;
   }

   /**
    * Remove item with specified value.
    * 
    * @param   item
    *          Item to remove.
    */
   public void remove(T item)
   {
      int index = indexOf(item);
      
      if (index >= 0)
         removeAt(index);
   }

   /**
    * Remove all items.
    */
   @Override
   public void clear()
   {
      super.clear();
      selected = null;
   }

   /**
    * Remove item at specified position.
    * 
    * @param   index
    *          Item index.
    */
   @Override
   public void removeAt(int index)
   {
      if (selected == get(index))
      {
         if (index == 0)
            selected = (size() == 1) ? null : get(1);
         else
            selected = get(index - 1);
      }

      super.removeAt(index);
   }

   /**
    * Get selected item.
    * 
    * @return  selected item.
    */
   public T selected()
   {
      return selected;
   }

   /**
    * Get selected index.
    * 
    * @return  selected index.
    */
   public int selectedIndex()
   {
      return selectedIndex;
   }

   /**
    * Set selected index.
    * 
    * @param   index
    *          Selected item index.
    */
   public void index(int index)
   {
      selectedIndex = index;
   }

   /**
    * Get selected item id.
    * 
    * @return  selected item id.
    */
   public int selectedId()
   {
      return selectedId;
   }

   /**
    * Set selected item if.
    * 
    * @param   id
    *          Selected item id.
    */
   public void id(int id)
   {
      selectedId = id;
   }

   /**
    * Get selected value.
    * 
    * @return  selected value.
    */
   public BaseDataType selectedValue()
   {
      return selectedValue;
   }

   /**
    * Select item with specified value.
    * 
    * @param   item
    *          Item value to select.
    */
   public void select(T item)
   {
      selected = item;
   }

   /**
    * Set selected index.
    * 
    * @param   index
    *          Selected item index.
    */
   public void value(BaseDataType value)
   {
      selectedValue = value;
   }
   
   /**
    * Calculate the index position of the current selection.
    *
    * @param    items
    *           The list of items to be used within this drop-down list.
    *
    * @return   The 0-based index position if there is a current selection
    *           in the given model or 0 if there is no selection.
    */
   public int calcSelection(ControlSetItem[] items)
   {
      if( selectedId > 0)
      {
         for(int i = 0; i < items.length; i++) {
            if( items[i].getId() == selectedId )
            {
               return selectedIndex >= 0 ? selectedIndex : i;
            }
         }
      }
      int idx = 0;
      if(selectedValue != null)
      {
         for (int i = 0; i < items.length; i++)
         {
            if( selectedValue.equals(items[i].getValue()))
            {
               idx = i;
               break;
            }
         }
      }
      return idx;
   }
   /**
    * Enforce the current selection.
    *
    * @param    items
    *           The list of items to be used within this drop-down list.
    */

   public void setSelection(ControlSetItem[] items) {
      int idx = calcSelection(items);
      
      if( 0 <= idx && idx < items.length) {
         ControlSetItem item = items[idx];
         index(idx);
         id(item.getId());
      }
   }
   
   
}
